function RUNME()
%%% project: emgr – EMpirical GRamian Framework Version 5.99
%%% version: 1.1 (2023-01-01)
%%% authors: C. Himpe (0000-0003-2194-6754)
%%% license: BSD 2-Clause License ( opensource.org/licenses/BSD-2-Clause )
%%% summary: Runs JAK/STAT parameter identification

%% Setup Solver

  global ODE;

  emgr_version = emgr('version')

  ODE = @rk45;

%% Compute Gramians

  dt = 0.001;
  Tf = 1.0;
  x0 = [1.3;0.0;0;0;0;2.8;165.0;0;0.34;0];

  flags = [0,0,0,0,1,0,0,0,2,1,0,0,0];
  param = 10.0.^(ones(23,1)*[-5,-1]);

  W1 = emgr(@vf,@of,[1,10,8],[dt,Tf],'i',param,flags,[],80.0,x0);
  W2 = emgr(@vf,@of,[1,10,8],[dt,Tf],'i',param,flags,[],20.0,x0);
  W3 = emgr(@vf,@of,[1,10,8],[dt,Tf],'i',param,flags,[],4.0,x0);
  W4 = emgr(@vf,@of,[1,10,8],[dt,Tf],'i',param,flags,[],0.0,x0);

  WI = W1{2} + W2{2} + W3{2} + W4{2};

%% Determine Identifiability

  [UI,DI,VI] = svd(WI);
  di = diag(DI);

  idx = 17;

  UI = abs(UI) ./ sum(abs(UI),2);

  IDENTIFIABLE_1_2_5_6_7_8_9_10_13_18_20_23 = sum(abs(UI([1,2,5,6,7,8,9,10,13,18,20,23],idx:end)),2)

  PRACT_UNIDENTIFIABLE_3_4_12_14_16_19 = sum(abs(UI([3,4,12,14,16,19],idx:end)),2)

  STRUC_UNIDENTIFIABLE_11_15_17_21_22 = sum(abs(UI([11,15,17,21,22],idx:end)),2)

%% Plot Results

  figure;
  h = semilogy(di,'linewidth',3);
  pbaspect([4 2 1]);
  xlim([0,numel(di)+1]);
  line([16,16], ylim, 'color','black');
  set(gca, 'YGrid', 'on', 'XGrid', 'off');
  xlabel('n-th Singular Value');
  ylabel('Magnitude');
  xticks([1:2:numel(di)]);
  saveas(gcf,'singval.pdf');
end

%% JAK/STAT Vector Field

function xdot = vf(x,u,p,t)

  c = [ 2.265; ...
       91.0];

  xdot = [-(p(5) * x(1)) + (p(6) * x(2)) - (p(1) * u(1) * c(1) * x(1)); ...
           (p(5) * x(1)) - (p(6) * x(2)); ...
           (p(2) * x(3) * (x(6)-2.8)) + (p(1) * u(1) * c(1) * x(1)); ...
          -(p(2) * x(3) * (x(6)-2.8)) - (p(3) * x(4)); ...
           (p(3) * x(4)) - (p(4) * x(5)); ...
          -(p(7) * x(3) * x(6)) ./ (1.0 + p(13) * x(1)) - (p(7) * x(4) * x(6)) ./ (1.0 + p(13) * x(1)) - (p(8) * c(2) * (x(6)-2.8)); ...

           (p(9) * x(7) * (x(6)-2.8)) - (c(2) * p(10) * (x(7)-165.0)); ...

          -(p(11) * (x(7)-165.0)); ...
          -(p(12) * c(1) * u(1) * x(9)); ...

           (p(14) * x(8)) ./ (p(15) + x(8)) - (p(16) * x(10))];
end

%% JAK/STAT Output Function

function y = of(x,u,p,t)

  y = [x(1) + x(3) + x(4); ...
       p(18) * (x(3) + x(4) + x(5) + 0.34 - x(9)); ...
       p(19) * (x(4) + x(5)); ...
       p(20) * (2.8 - x(6)); ...
       p(21) * x(8); ...
       p(17) * p(22) * x(8) / p(11); ...
       x(10); ...
       165.0 - x(7)];
end

%% Solver Wrapper

function y = rk45(f,g,t,x0,u,p)

  x0(2) = x0(2) + p(23);

  [S,x] = ode45(@(t,x) f(x,u(t),p,t),[0,t(2)],x0,odeset('InitialStep',t(1)));
  z = cell2mat(arrayfun(@(k) g(x(k,:)',u(S(k)),p,S(k)),1:numel(S),'UniformOutput',false));
  y = interp1(S,z',(0:t(1):t(2)))';
end

